// -*-Mode: C++;-*- 
/*
 * $Id: tabledescr.h,v 2.7 1999/08/07 19:33:12 ral Exp $
 *
 * TableDescriptor - Class
 * it will be used to wrap the sql-entries
 * into a class which can use with various forms of sql-server
 */

#ifndef table_descr_inc
#define table_descr_inc

#include <vector>
#include <string>
#include "liste.h"
#include "stringlist.h"

class CKeyDescriptor
{
public:
    enum Types {NONE,PRIMARY,UNIQUE,INDEX,FOREIGN};
private:
    std::string name;
    stringIlist ColumnsList;
    Types type;
    bool compound;

public:
    CKeyDescriptor(){type = NONE;compound = false;}
    CKeyDescriptor(const CKeyDescriptor&);
    virtual ~CKeyDescriptor(){};
    
    //compares name against a string
    int operator == (const char*what);
    int operator == (const std::string&what);

    //compares against another descriptor
    int operator == (const CKeyDescriptor&);
    CKeyDescriptor&operator= (const CKeyDescriptor&old);

    void set_name(const char*n_name){name = n_name?n_name:"";}
    void set_name(const std::string&n_name){name = n_name;}
    const char*get_name(){return name.c_str();}

    void setType(Types nType){type = nType;}
    Types getType(){return type;}

    int add_column_name(const char*column);
    void remove_column_name(const char*column);
    bool contains_column_name(const char*column);
    int add_column_name(const std::string&column);
    void remove_column_name(const std::string&column);
    bool contains_column_name(const std::string&column);

    unsigned int column_count(){return ColumnsList.size();}

    const stringIlist get_ColList(){return ColumnsList;}
    const bool is_compound()const{return compound;}
    void set_compound(bool com = false){compound = com;}
};

inline int CKeyDescriptor::operator ==(const char*what)
{
    // cause all fields in mysql are caseinsensitive
    return ( strcasecmp(name.c_str(),what) == 0);
}

inline int CKeyDescriptor::operator ==(const CKeyDescriptor&what)
{
    return (strcasecmp(name.c_str(), what.name.c_str()) == 0);
}

inline int CKeyDescriptor::add_column_name(const std::string&column)
{
    return (column.size() == 0)?0:add_column_name(column.c_str());
}

inline void CKeyDescriptor::remove_column_name(const std::string&column)
{
    if (column.size())
	remove_column_name(column.c_str());
}

inline bool CKeyDescriptor::contains_column_name(const std::string&column)
{
    return column.size()>0?contains_column_name(column.c_str()):false;
}

typedef my_vector<CKeyDescriptor> KeyList;
typedef KeyList::iterator KeyList_iterator;

class CColumnDescriptor 
{
    string name;
    string Def;
    string table;

    int FieldType;
    int MaxLength;
    int MaxDecimals;
    
    bool AutoIncrement;
    bool KeyField;
    bool Binary;
    bool TimeStamp;
    bool ZeroFill;
    bool NotNull;
    bool Blob;
    bool PartOfMultiple;
    bool Numeric;
    bool UniqueValue;
    bool Unsigned;

    stringIlist Enums;

public:
    CColumnDescriptor();
    virtual ~CColumnDescriptor(){}
    CColumnDescriptor(const CColumnDescriptor&old);
    CColumnDescriptor&operator=(const CColumnDescriptor&old);

    void set_name(const char*Name){name = Name?Name:"";}
    void set_name(const std::string&Name){name=Name;}
    const char*get_name(){return name.c_str();}

    void set_default(const char*def){Def = def?def:"";}
    void set_default(const std::string&def){Def = def;}
    const char*get_default(){return Def.c_str();}

    void set_table(const char*t){table = t?t:"";}
    void set_table(const std::string&t){table=t;}
    const char*get_table(){return table.c_str();}
    
    void set_FieldType(int type){FieldType = type;}
    int get_FieldType(){return FieldType;}

    const stringIlist&enum_list(){return Enums;}
    void enum_list(const stringIlist&list){Enums = list;}
    
    bool contains_enum(std::string&what){return Enums.find(what)!=0;}
    bool contains_enum(const char*what){return what?Enums.find(std::string(what)):false;}
    void insert_enum(std::string&what){if (!contains_enum(what))Enums.push_back(what);}
    void insert_enum(const char*what){if (what&&!contains_enum(what))Enums.push_back(std::string(what));}
    void remove_enum(const std::string&what);
    void remove_enum(const char*what){if (what) remove_enum(std::string(what));}

    void set_Unsigned(bool how){Unsigned = how;}
    bool is_Unsigned(){return Unsigned;}
    bool is_AutoIncrement(){return AutoIncrement;}
    void set_AutoIncrement(bool how){AutoIncrement = how;}
    bool is_KeyField(){return KeyField;}
    void set_KeyField(bool how){KeyField = how;}
    bool is_Binary(){return Binary;}
    void set_Binary(bool how){Binary = how;}
    bool is_TimeStamp(){return TimeStamp;}
    void set_TimeStamp(bool how){TimeStamp = how;}
    bool is_ZeroFill(){return ZeroFill;}
    void set_ZeroFill(bool how){ZeroFill = how;}
    bool is_NotNull(){return NotNull;}
    void set_NotNull(bool how){NotNull = how;}
    bool is_Blob(){return Blob;}
    void set_Blob(bool how){Blob = how;}
    bool is_PartOfMultiKey(){return PartOfMultiple;}
    void set_PartOfMultiKey(bool how){PartOfMultiple = how;}
    bool is_Numeric(){return Numeric;}
    void set_Numeric(bool how){Numeric = how;}
    bool is_Unique(){return UniqueValue;}
    void set_Unique(bool how){UniqueValue = how;}
    int get_MaxLength(){return MaxLength;}
    void set_MaxLength(int Length){MaxLength = Length;}
    int get_MaxDecimals(){return MaxDecimals;}
    void set_MaxDecimals(int Dec){MaxDecimals = Dec;}

    //compares name against a string
    int operator == (const char*what);
    int operator == (const std::string&what);
    //compares against another descriptor
    int operator == (const CColumnDescriptor&what);
    
    int operator != (const CColumnDescriptor&what){return !(*this==what);}
protected:
    void copy_old(const CColumnDescriptor*old);
};

inline int CColumnDescriptor::operator ==(const char*what)
{
    return what?(strcasecmp(name.c_str(),what) == 0):0;
}

inline int CColumnDescriptor::operator ==(const std::string&what)
{
    return (strcasecmp(name.c_str(),what.c_str()) == 0);
}

inline int CColumnDescriptor::operator ==(const CColumnDescriptor&what)
{
    return (strcasecmp(name.c_str(),what.name.c_str()) == 0);
}

typedef my_vector<CColumnDescriptor> ColumnList;
typedef ColumnList::iterator ColumnList_iterator;

#endif
